from ._v2 import EtsyV2 as Etsy
from .etsy_env import EtsyEnvProduction


__version__ = '0.4.0'
__author__ = 'Sean Scheetz'
__copyright__ = 'Copyright 2010, Etsy Inc.'
__license__ = 'GPL v3'
__email__ = 'file_an_issue_on_github@gmail.com'

