class EtsyEnvProduction(object):
    request_token_url = 'https://openapi.etsy.com/v2/oauth/request_token'
    access_token_url = 'https://openapi.etsy.com/v2/oauth/access_token'
    signin_url = 'https://www.etsy.com/oauth/signin'
    api_url = 'https://openapi.etsy.com/v2'
